<?php
/**************************************************************************\
* eGroupWare                                                               *
* http://www.egroupware.org                                                *
* --------------------------------------------                             *
*  This program is free software; you can redistribute it and/or modify it *
*  under the terms of the GNU General Public License as published by the   *
*  Free Software Foundation; either version 2 of the License, or (at your  *
*  option) any later version.                                              *
\**************************************************************************/

require_once WF_LIB . SEP . 'BaseFactory.php';

/**
 * Implementa o design pattern factory para os processos Workflow
 * @author Sidnei Augusto Drovetto Jr. - drovetto@gmail.com
 * @author Carlos Eduardo Nogueira Gonalves
 * @version 1.0
 * @package Workflow
 * @license http://www.gnu.org/copyleft/gpl.html GPL
 */
class ProcessFactory extends BaseFactory {

	/**
	 * @var array $_cache Array de cache dos objetos gerados
	 * @access private
	 */
	private $_cache;

	/**
	 * @var string $_classPath Armazena o caminho para onde esto as classes a serem instanciadas
	 * @access private
	 */
	private $_classPath;

	/**
	 * @var string $_preffix Armazena o prefixo do nome dos arquivos das classes
	 * @access private
	 */
	private $_preffix;

	/**
	 * @var string $_suffix Armazena o sufixo do nome dos arquivos das classes
	 * @access private
	 */
	private $_suffix;

	/**
	 * Construtora da class ProcessFactory
	 * @return object
	 * @access public
	 */
	public function ProcessFactory(){
		$this->_cache		= array();
		$this->_classPath	= PHPGW_SERVER_ROOT . SEP . 'workflow' . SEP . 'inc' . SEP . 'local' . SEP . 'classes';
		$this->_preffix		= 'class.';
		$this->_suffix		= '.php';
	}

	/**
	 * Importa os arquivos da classe
	 * @param string $input Nome da classe
	 * @access private
	 * @return void
	 */
	protected function _import($input) {

		$file = $this->_classPath . '/' . $this->_preffix . $input . $this->_suffix;

		if (@file_exists($file)){
			require_once $file;
		} else {
			trigger_error("O arquivo " . $file . " no existe.", E_USER_ERROR);
		}

	}

	/**
	 * Armazena referncias aos objetos recm criados
	 * @param string $key O nome da classe
	 * @param object &$item A instncia da classe
	 * @return boolean True se for salvo. False caso contrrio, ou $item no  um objeto
	 * @access private
	 */
	protected function _setEntry($key, &$item) {
		if(!is_object($item))
			return false;

		$this->_cache[] =& $item;
		return true;
	}

	/**
	 * Retorna a referncia ao objeto armazenado
	 * @param string $key Nome da classe
	 * @return object
	 * @access private
	 */
	protected function &_getEntry($key) {
		return $this->_cache[$key];
	}

	/**
	 * Verifica se uma determinada classe possui objeto armazenado no cache da Factory
	 * @param string $key Nome da classe
	 * @return boolean
	 * @access protected
	 */
	protected function _isEntry($key) {
		return ($this->_getEntry($key) !== null);
	}

	/**
	 * Retorna objetos armazenados ou cria novos objetos utilizando o pattern singleton
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	public function &getInstance() {
		if (func_num_args() > 0) {
			$args = func_get_args();
			if ($this->_isEntry($args[0])) {
				return($this->_getEntry($args[0]));
			} else {
				$instance = call_user_func_array(array(&$this, 'newInstance'), $args);
				$this->_setEntry($args[0], &$instance);
				return $instance;
			}
		}
	}

	/**
	 * Cria novos objetos
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	public function &newInstance() {
		if (func_num_args() > 0) {

			$args = func_get_args();
			$className = array_shift($args);

			$this->_import($className);

			$reflectionObj = new ReflectionClass($className);
			return $reflectionObj->newInstanceArgs($args);
		}
	}
}
?>
