<?php
/**************************************************************************\
* eGroupWare                                                               *
* http://www.egroupware.org                                                *
* --------------------------------------------                             *
*  This program is free software; you can redistribute it and/or modify it *
*  under the terms of the GNU General Public License as published by the   *
*  Free Software Foundation; either version 2 of the License, or (at your  *
*  option) any later version.                                              *
\**************************************************************************/

/**
 * Implementa o design pattern factory para o mdulo Workflow
 * @author Carlos Eduardo Nogueira Gonalves
 * @author Sidnei Augusto Drovetto Jr. - drovetto@gmail.com
 * @version 1.1
 * @package Workflow
 * @license http://www.gnu.org/copyleft/gpl.html GPL
 */
abstract class BaseFactory {

	/**
	 * Importa os arquivos da classe
	 * @param string $input Nome da classe
	 * @access private
	 * @return void
	 */
	abstract protected function _import($input);

	/**
	 * Armazena referncias aos objetos recm criados
	 * @param string $key O nome da classe
	 * @param object &$item A instncia da classe
	 * @return boolean True se for salvo. False caso contrrio, ou $item no  um objeto
	 * @access private
	 */
	abstract protected function _setEntry($key, &$item);

	/**
	 * Retorna a referncia ao objeto armazenado
	 * @param string $key Nome da classe
	 * @return object
	 * @access private
	 */
	abstract protected function &_getEntry($key);

	/**
	 * Verifica se uma determinada classe possui objeto armazenado no cache da Factory
	 * @param string $key Nome da classe
	 * @return boolean
	 * @access public
	 */
	abstract protected function _isEntry($key);

	/**
	 * Retorna objetos armazenados ou cria novos objetos utilizando o pattern singleton
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	abstract public function &getInstance();

	/**
	 * Cria novos objetos
	 * @param string $className Nome da classe
	 * @return object
	 * @access public
	 */
	abstract public function &newInstance();

}
?>
